package cn.com.qmth.stmms.api.controller.admin;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.admin.utils.ExportInspectExcel;
import cn.com.qmth.stmms.admin.vo.InspectedSubjectVO;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.ExamStudentVo;
import cn.com.qmth.stmms.biz.exam.bean.InspectWorkVo;
import cn.com.qmth.stmms.biz.exam.bean.InspectedSubjectChatVo;
import cn.com.qmth.stmms.biz.exam.bean.InspectedSubjectStatusVo;
import cn.com.qmth.stmms.biz.exam.bean.NextRoundCheckVo;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.OperationLog;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.query.InspectQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.InspectHistoryService;
import cn.com.qmth.stmms.biz.exam.service.InspectedService;
import cn.com.qmth.stmms.biz.exam.service.OperationLogService;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.exception.StatusException;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "成绩复核")
@Controller("adminInspectedController")
@RequestMapping("/api/admin/exam/inspected")
public class InspectedController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(InspectedController.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private InspectedService inspectedService;

    @Autowired
    private InspectHistoryService inspectHistoryService;

    @Autowired
    private LockService lockService;

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private OperationLogService logService;

    @Value("${slice.split.config}")
    private String splitConfig;

    public static final String UN_SELECTIVE_SCORE = "-1";

    private static final String DEFAULT_SECRET_NUMBER = "***";

    @ApiOperation(value = "考试是否结束")
    @RequestMapping(value = "exam/finish", method = RequestMethod.POST)
    @ResponseBody
    public Boolean examFinish() {
        int examId = getSessionExamId();
        Exam exam = examService.findById(examId);
        return checkFinish(exam);
    }

    @ApiOperation(value = "成绩复核进度图表")
    @RequestMapping(value = "chat", method = RequestMethod.POST)
    @ResponseBody
    public InspectedSubjectChatVo chat(ExamSubjectSearchQuery query) {
        int examId = getSessionExamId();
        query.setExamId(examId);
        Set<String> unFinishSet = studentService.findSubjectUnInspectedByExamId(examId);
        ExamStudentSearchQuery sQuery = new ExamStudentSearchQuery();
        sQuery.setExamId(examId);
        sQuery.setUpload(true);
        sQuery.setAbsent(false);
        sQuery.setBreach(false);
        sQuery.setInspected(true);
        long markedCount = studentService.countByQuery(sQuery);
        sQuery.setInspected(null);
        sQuery.addStatus(SubjectiveStatus.MARKED);
        long total = studentService.countByQuery(sQuery);
        Set<String> hasTask = studentService.findSubjectHasTaskByExamId(examId);
        long hasTaskCount = 0;
        if (hasTask != null) {
            hasTaskCount = hasTask.size();
        }
        InspectedSubjectChatVo vo = new InspectedSubjectChatVo();
        vo.setFinishCount(hasTaskCount - unFinishSet.size());
        vo.setMarkedCount(markedCount);
        vo.setUnFinishCount(Long.valueOf(unFinishSet.size()));
        vo.setUnMarkedCount(total - markedCount);
        return vo;
    }

    @Logging(menu = "成绩复核进度查询", type = LogType.QUERY)
    @ApiOperation(value = "成绩复核进度查询")
    @RequestMapping(value = "info", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<InspectedSubjectVO> info(ExamSubjectSearchQuery query) {
        return infoPage(query);
    }

    private PageResult<InspectedSubjectVO> infoPage(ExamSubjectSearchQuery query) {
        ApiUser wu = getApiUser();
        int examId = getSessionExamId();
        query.setExamId(examId);
        Set<String> unFinishSet = studentService.findSubjectUnInspectedByExamId(examId);
        Set<String> hasTaskSet = studentService.findSubjectHasTaskByExamId(examId);
        if (query.getFinished() != null) {
            String subjectCodeIn = StringUtils.join(unFinishSet, ",");
            String hasTaskSetIn = StringUtils.join(hasTaskSet, ",");
            if (query.getFinished()) {
                query.setCodeNotIn(subjectCodeIn);
                query.setCodeIn(hasTaskSetIn);
            } else {
                query.setCodeIn(subjectCodeIn);
            }
        }
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (StringUtils.isBlank(query.getCode()) && wu.getRole() != Role.SCHOOL_ADMIN) {
            List<String> subjectCodeIn = new ArrayList<String>();
            subjectList.forEach(e -> subjectCodeIn.add(e.getCode()));
            subjectCodeQuery(query, subjectCodeIn);
        }
        query = subjectService.findByQuery(query);
        if (unFinishSet.isEmpty() && query.getFinished() != null && !query.getFinished()) {
            query.setResult(new ArrayList<ExamSubject>());
        }

        List<InspectedSubjectVO> list = new LinkedList<InspectedSubjectVO>();
        for (ExamSubject subject : query.getResult()) {
            InspectedSubjectVO vo = new InspectedSubjectVO();
            vo.setSubject(subject);
            vo.setSubjectText(subject.getCode() + "-" + subject.getName());
            vo.setSelectiveText(subject.isSelective() ? "是" : "否");
            ExamStudentSearchQuery sQuery = new ExamStudentSearchQuery();
            sQuery.setExamId(examId);
            sQuery.setUpload(true);
            sQuery.setAbsent(false);
            sQuery.setBreach(false);
            sQuery.setSubjectCode(subject.getCode());
            long totalPaperCount = studentService.countByQuery(sQuery);
            vo.setTotalPaperCount(totalPaperCount);
            sQuery.setInspected(true);
            long inspectedCount = studentService.countByQuery(sQuery);
            vo.setInspectedCount(inspectedCount);
            sQuery.addStatus(SubjectiveStatus.MARKED);
            sQuery.setInspected(null);
            long totalCount = studentService.countByQuery(sQuery);
            vo.setTotalCount(totalCount);
            vo.setLeftCount(totalCount - inspectedCount);
            if (vo.getLeftCount() < 0) {
                vo.setLeftCount(0);
            }
            String percent = totalPaperCount > 0
                    ? (new DecimalFormat("####.###").format(inspectedCount * 100.0 / totalPaperCount) + "%")
                    : "0%";
            vo.setPercent(percent);
            if (unFinishSet.contains(vo.getSubject().getCode())) {
                vo.setFinishCount(vo.getSubject().getInspectRound() - 1);
            } else {
                if (totalCount == 0) {
                    vo.setFinishCount(vo.getSubject().getInspectRound() - 1);
                } else {
                    vo.setFinishCount(vo.getSubject().getInspectRound());
                }
            }
            list.add(vo);
        }
        return PageUtil.of(list, query);
    }

    @ApiOperation(value = "成绩复核进度导出")
    @RequestMapping(value = "info/export", method = RequestMethod.POST)
    @ResponseBody
    public void infoExport(ExamSubjectSearchQuery query, HttpServletResponse response) {
        String fileName = "成绩复核进度数据.xlsx";
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        PageResult<InspectedSubjectVO> page = infoPage(query);
        try {
            new ExportExcel("成绩复核进度数据", InspectedSubjectVO.class).setDataList(page.getResult())
                    .write(response, fileName).dispose();
        } catch (IOException e) {
            throw new StatusException("成绩复核进度数据导出出错", e);
        }
    }

    @ApiOperation(value = "科目状态信息")
    @RequestMapping(value = "subject/status", method = RequestMethod.POST)
    @ResponseBody
    public InspectedSubjectStatusVo subjectStatus(InspectQuery query) {
        int examId = getSessionExamId();
        Exam exam = examService.findById(examId);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            throw new StatusException("多媒体阅卷无此功能权限");
        }
        Integer inspectCount = inspectedService.needInspectCount(query);

        InspectedSubjectStatusVo ret = new InspectedSubjectStatusVo();
        ret.setNeedInspectCount(inspectCount);
        if (StringUtils.isNotBlank(query.getSubjectCode())) {
            ExamSubject es = subjectService.find(examId, query.getSubjectCode());
            Boolean inspectFinish = inspectedService.inspectFinish(examId, query.getSubjectCode(),
                    es.getInspectRound());
            ret.setShowNextInspectMsg(inspectFinish);
            if (!exam.getInspectRoundLimit()) {
                ret.setShowNextInspect(true);
            } else {
                if (inspectFinish) {
                    ret.setShowNextInspect(true);
                } else {
                    ret.setShowNextInspect(false);
                }
            }
        }
        return ret;
    }

    @ApiOperation(value = "复核轮数列表")
    @RequestMapping(value = "round/list", method = RequestMethod.POST)
    @ResponseBody
    public List<Integer> roundList(InspectQuery query) {
        int examId = getSessionExamId();
        return inspectHistoryService.findInspectRound(examId);
    }

    @ApiOperation(value = "全卷复核查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ExamStudentVo> list(InspectQuery query) {
        int examId = getSessionExamId();
        Exam exam = examService.findById(examId);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            throw new StatusException("多媒体阅卷无此功能权限");
        }
        ApiUser wu = getApiUser();
        query.setExamId(examId);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (StringUtils.isBlank(query.getSubjectCode()) && wu.getRole() != Role.SCHOOL_ADMIN) {
            List<String> subjectCodeIn = new ArrayList<String>();
            subjectList.forEach(e -> subjectCodeIn.add(e.getCode()));
            query.setSubjectCodeIn(StringUtils.join(subjectCodeIn, ","));
        }
        if (wu.getRole() != Role.SCHOOL_ADMIN && subjectList != null && subjectList.size() > 0
                && query.getSubjectCode() == null) {
            query.setSubjectCode(subjectList.get(0).getCode());
        }
        List<ExamStudentVo> list = inspectedService.findByQuery(query);
        for (ExamStudentVo e : list) {
            e.setSubjectiveScoreList(e.getSubjectiveScoreList().replace(UN_SELECTIVE_SCORE, "/"));
            e.setSubjectText(e.getSubjectCode() + "-" + e.getSubjectName());
            if (e.getInspectTagFlagged() == null) {
                e.setInspectTagFlaggedText("否");
            } else {
                e.setInspectTagFlaggedText(e.getInspectTagFlagged() ? "是" : "否");
            }
        }
        int maxInspectRound = 1;
        if (list != null && list.size() > 0) {
            Set<String> subjectCodes = list.stream().map(e -> e.getSubjectCode()).collect(Collectors.toSet());
            maxInspectRound = subjectService.findMaxInspectRound(examId, subjectCodes);
            inspectedService.fillInspectHistoryMap(list, maxInspectRound);
        }
        Integer totalCount = inspectedService.countByQuery(query);
        query.setResult(list);
        query.setTotalCount(totalCount);
        return PageUtil.of(query);
    }

    private boolean checkFinish(Exam exam) {
        List<ExamSubject> list = subjectService.list(exam.getId());
        for (ExamSubject subject : list) {
            String meaages = subjectService.enableExport(exam, subject.getCode());
            if (meaages != null) {
                return false;
            }
        }
        return true;
    }

    @ApiOperation(value = "再次复核操作前校验")
    @RequestMapping(value = "nextround/check", method = RequestMethod.POST)
    @ResponseBody
    public NextRoundCheckVo nextroundCheck(@RequestParam String subjectCode) {
        NextRoundCheckVo obj = new NextRoundCheckVo();
        // status -1 错误，0需确认，1通过
        int examId = getSessionExamId();
        lockService.waitlock(LockType.EXAM_SUBJECT, examId, subjectCode);
        try {
            Exam exam = examService.findById(examId);
            ExamSubject es = subjectService.find(examId, subjectCode);
            if (exam.getInspectRoundLimit()) {
                if (!inspectedService.inspectFinish(examId, subjectCode, es.getInspectRound())) {
                    obj.setStatus(-1);
                    obj.setMessage("该科目第" + es.getInspectRound() + "次复核还未完成");
                    return obj;
                } else {
                    String msg = es.getCode() + "-" + es.getName() + ",复核进度已到100%，是否确定开启新一轮复核?";
                    obj.setMessage(msg);
                    obj.setStatus(0);
                    return obj;
                }
            } else {
                ExamStudentSearchQuery sQuery = new ExamStudentSearchQuery();
                sQuery.setExamId(examId);
                sQuery.setUpload(true);
                sQuery.setAbsent(false);
                sQuery.setBreach(false);
                sQuery.setSubjectCode(subjectCode);
                sQuery.setInspected(true);
                long inspectedCount = studentService.countByQuery(sQuery);
                if (inspectedCount == 0) {
                    obj.setStatus(-1);
                    obj.setMessage("本轮无复核记录，无法开启再次复核");
                    return obj;
                }
                sQuery.setInspected(null);
                long totalCount = studentService.countByQuery(sQuery);
                if (inspectedCount == totalCount) {
                    String msg = es.getCode() + "-" + es.getName() + ",复核进度已到100%，是否确定开启新一轮复核?";
                    obj.setMessage(msg);
                    obj.setStatus(0);
                    return obj;
                } else {
                    String percent = totalCount > 0
                            ? (new DecimalFormat("####.###").format(inspectedCount * 100.0 / totalCount) + "%")
                            : "0%";
                    String msg = "本轮复核进度为" + percent + "，还有" + (totalCount - inspectedCount) + "份未完成复核，是否进入下一轮复核?";
                    obj.setStatus(0);
                    obj.setMessage(msg);
                    return obj;
                }
            }
        } finally {
            lockService.unlock(LockType.EXAM_SUBJECT, examId, subjectCode);
        }
    }

    @ApiOperation(value = "再次复核")
    @Logging(menu = "再次复核", type = LogType.UPDATE)
    @RequestMapping(value = "nextround", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage nextround(@RequestParam String subjectCode) {
        int examId = getSessionExamId();
        lockService.waitlock(LockType.EXAM_SUBJECT, examId, subjectCode);
        try {
            Exam exam = examService.findById(examId);
            ExamSubject es = subjectService.find(examId, subjectCode);
            StringBuilder sb = new StringBuilder();
            sb.append("科目代码：" + subjectCode);
            sb.append(" | 开启第" + (es.getInspectRound() + 1) + "轮");
            sb.append(" | 上轮进度");
            ExamStudentSearchQuery sQuery = new ExamStudentSearchQuery();
            sQuery.setExamId(examId);
            sQuery.setUpload(true);
            sQuery.setAbsent(false);
            sQuery.setBreach(false);
            sQuery.setSubjectCode(subjectCode);
            sQuery.setInspected(true);
            if (exam.getInspectRoundLimit()) {
                if (!inspectedService.inspectFinish(examId, subjectCode, es.getInspectRound())) {
                    throw new StatusException("该科目第" + es.getInspectRound() + "次复核还未完成");
                } else {
                    long inspectedCount = studentService.countByQuery(sQuery);
                    sQuery.setInspected(null);
                    long totalCount = studentService.countByQuery(sQuery);
                    sb.append(" 总量：" + totalCount);
                    sb.append(" 已复核：" + inspectedCount);
                    sb.append(" 未复核：" + (totalCount - inspectedCount));
                    subjectService.nextInspectRound(examId, subjectCode);

                    RequestUtils.setLog(sb.toString());
                    return resultOk();
                }
            } else {
                long inspectedCount = studentService.countByQuery(sQuery);
                if (inspectedCount == 0) {
                    throw new StatusException("本轮无复核记录，无法开启再次复核");
                }
                sQuery.setInspected(null);
                long totalCount = studentService.countByQuery(sQuery);
                sb.append(" 总量：" + totalCount);
                sb.append(" 已复核：" + inspectedCount);
                sb.append(" 未复核：" + (totalCount - inspectedCount));
                subjectService.nextInspectRound(examId, subjectCode);
                RequestUtils.setLog(sb.toString());
                return resultOk();
            }
        } finally {
            lockService.unlock(LockType.EXAM_SUBJECT, examId, subjectCode);
        }
    }

    @ApiOperation(value = "取消复核")
    @RequestMapping(value = "batch-cancel", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage batchCancel(@RequestParam List<Integer> studentIds) {
        ApiUser wu = getApiUser();
        for (Integer id : studentIds) {
            ExamStudent student = studentService.findById(id);
            if (student != null && subjectCheck(student.getSubjectCode(), wu)) {
                try {
                    lockService.watch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    lockService.waitlock(LockType.STUDENT, student.getId());
                    if (student.getInspected() && inspectedService.cancelByStudent(id)) {
                        OperationLog log = new OperationLog();
                        log.setCreateTime(new Date());
                        log.setMenu("取消复核");
                        log.setType(LogType.UPDATE);
                        log.setMarker(false);
                        log.setOperatorId(wu.getUser().getId());
                        log.setIpAddress(wu.getUser().getLastLoginIp());
                        log.setLoginName(wu.getUser().getLoginName());
                        int examId = getSessionExamId();
                        log.setExamId(examId > 0 ? examId : null);
                        log.setSchoolId(wu.getUser().getSchoolId());
                        log.setDescription("科目代码：" + student.getSubjectCode() + " 准考证号：" + student.getExamNumber()
                                + " 密号：" + student.getSecretNumber());
                        logService.save(log);
                    }
                } catch (Exception e) {
                    log.error("back inspected error", e);
                } finally {
                    lockService.unlock(LockType.STUDENT, student.getId());
                    lockService.unwatch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                }
            }
        }
        return resultOk();
    }

    private void subjectCodeQuery(ExamSubjectSearchQuery query, List<String> subjectCodeIn) {
        if (StringUtils.isEmpty(query.getCodeIn())) {
            query.setCodeIn(StringUtils.join(subjectCodeIn, ","));
        } else {
            Set<String> set = new HashSet<>();
            for (String c : query.getCodeIn().split(",")) {
                set.add(c);
            }
            set.addAll(subjectCodeIn);
            query.setCodeIn(StringUtils.join(set, ","));
        }
    }

    @ApiOperation(value = "全卷复核导出")
    @RequestMapping(value = "list/export", method = RequestMethod.POST)
    @ResponseBody
    public void exportListFile(HttpServletResponse response, InspectQuery query) {
        int examId = getSessionExamId();
        Exam exam = examService.findById(examId);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            throw new StatusException("多媒体阅卷无此功能权限");
        }
        ApiUser wu = getApiUser();
        query.setExamId(examId);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (StringUtils.isBlank(query.getSubjectCode()) && wu.getRole() != Role.SCHOOL_ADMIN) {
            List<String> subjectCodeIn = new ArrayList<String>();
            subjectList.forEach(e -> subjectCodeIn.add(e.getCode()));
            query.setSubjectCodeIn(StringUtils.join(subjectCodeIn, ","));
        }
        query.setExport(true);
        List<ExamStudentVo> list = inspectedService.findByQuery(query);
        for (ExamStudentVo e : list) {
            e.setSubjectiveScoreList(e.getSubjectiveScoreList().replace(UN_SELECTIVE_SCORE, "/"));
            if (exam.isForbiddenInfo() && !Role.SCHOOL_ADMIN.equals(wu.getRole())) {
                e.setSecretNumber(DEFAULT_SECRET_NUMBER);
            }
            e.setSubjectText(e.getSubjectCode() + "-" + e.getSubjectName());
            if (e.getInspectTagFlagged() == null) {
                e.setInspectTagFlaggedText("否");
            } else {
                e.setInspectTagFlaggedText(e.getInspectTagFlagged() ? "是" : "否");
            }
        }
        int maxInspectRound = 1;
        if (list != null && list.size() > 0) {
            Set<String> subjectCodes = list.stream().map(e -> e.getSubjectCode()).collect(Collectors.toSet());
            maxInspectRound = subjectService.findMaxInspectRound(examId, subjectCodes);
            inspectedService.fillInspectHistoryMap(list, maxInspectRound);
        }
        try {
            String fileName = "复核列表.xlsx";
            List<String> headerList = getInspectHeader(maxInspectRound);
            ExportInspectExcel excel = new ExportInspectExcel("复核列表", headerList, ExamStudentVo.class);
            excel.setDataList(list);
            excel.write(response, fileName).dispose();
        } catch (Exception e) {
            throw new StatusException("导出数据失败", e);
        }
    }

    private List<String> getInspectHeader(int maxInspectRound) {
        List<String> ret = new ArrayList<>();
        for (int i = 1; i <= maxInspectRound; i++) {
            ret.add("复核人" + i);
            ret.add("复核时间" + i);
        }
        return ret;
    }

    @ApiOperation(value = "复核工作量导出")
    @RequestMapping(value = "work/export", method = RequestMethod.POST)
    @ResponseBody
    public void exportWorkFile(HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        Integer examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            throw new StatusException("多媒体阅卷无此功能权限");
        }
        try {
            List<InspectWorkVo> list = inspectHistoryService.findInspectWork(examId);
            String fileName = "复核员工作量.xlsx";
            new ExportExcel("复核员工作量", InspectWorkVo.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            throw new StatusException("导出数据失败", e);
        }
    }

    @ApiOperation(value = "复核标记或取消标记")
    @RequestMapping(value = "tag-flag", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage tagFlag(@RequestParam Integer studentId, @RequestParam Boolean flag) {
        ApiUser wu = getApiUser();
        studentService.inspectTagFlag(studentId, flag, wu.getUser().getId());
        return resultOk();
    }

}
