package cn.com.qmth.stmms.api.controller.admin;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.google.common.collect.Lists;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.admin.vo.CollationLabelVO;
import cn.com.qmth.stmms.admin.vo.ExamStudentVO;
import cn.com.qmth.stmms.admin.vo.UploadStudentVO;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.config.service.impl.SystemCache;
import cn.com.qmth.stmms.biz.exam.model.*;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.ImportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONObject;

@Api(tags = "考生管理")
@Controller("adminStudentController")
@RequestMapping("/api/admin/student")
public class StudentController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(StudentController.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    @Autowired
    private MarkService markService;

    @Autowired
    private ExamPackageService packageService;

    @Autowired
    private FileService fileService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private CollationLabelService collationLabelService;

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    private static final String PAPER_TYPES_REGEX = "[a-zA-Z#]";

    @ApiOperation(value = "查询考生")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    @Logging(menu = "查询考生", type = LogType.QUERY)
    public PageResult<ExamStudent> list(HttpServletRequest request, ExamStudentSearchQuery query) {
        int examId = getSessionExamId(request);
        User user = RequestUtils.getApiUser(request).getUser();
        List<CollationLabel> collationLabel = collationLabelService.list(examId);
        Map<String, CollationLabel> collationLabelMap = collationLabel.stream()
                .collect(Collectors.toMap(CollationLabel::getCode, Function.identity()));
        query.setExamId(examId);
        if (query.getBatchCode() != null) {
            query.setBatchCode(StringUtils.upperCase(query.getBatchCode()));
        }
        query.addSort("id", Sort.Direction.ASC);
        query = studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            student.setSubject(subjectService.find(student.getExamId(), student.getSubjectCode()));
            buildSheetUrl(student);
            buildPackageUrl(student);
            if (StringUtils.isNotBlank(student.getCollationLabelCode())) {
                student.setCollationLabel(collationLabelMap.get(student.getCollationLabelCode()).getName());
            }
        }
        return PageUtil.of(query);
    }

    private void buildSheetUrl(ExamStudent student) {
        student.setSheetUrls(
                fileService.getSheetUris(student.getExamId(), student.getExamNumber(), 1, student.getSheetCount()));
    }

    private void buildPackageUrl(ExamStudent student) {
        if (StringUtils.isNotBlank(student.getPackageCode())) {
            ExamPackage ep = packageService.find(student.getExamId(), student.getPackageCode());
            if (ep != null && ep.getPicCount() > 0) {
                student.setPackageUrls(fileService.getPackageUris(ep.getExamId(), ep.getCode(), 1, ep.getPicCount()));
            }
        }
    }

    @ApiOperation(value = "学院查询")
    @RequestMapping(value = "/college/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> collegeList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return studentService.findDistinctCollege(examId);
    }

    @ApiOperation(value = "班级查询")
    @RequestMapping(value = "/className/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> classNameList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return studentService.findDistinctClassName(examId);
    }

    @ApiOperation(value = "考点查询")
    @RequestMapping(value = "/examSite/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> examSiteList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return studentService.findDistinctExamSite(examId);
    }

    @ApiOperation(value = "获取考生")
    @RequestMapping(value = "/find", method = RequestMethod.POST)
    @ResponseBody
    @Logging(menu = "获取考生", type = LogType.QUERY)
    public ExamStudent findOne(HttpServletRequest request, @RequestParam Integer id) {
        return studentService.findById(id);
    }

    @ApiOperation(value = "新增考生")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ResponseBody
    @Logging(menu = "新增考生", type = LogType.ADD)
    public ResultMessage save(HttpServletRequest request, ExamStudent student) {
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        ExamSubject subject = subjectService.find(examId, student.getSubjectCode());
        if (subject == null) {
            throw new StatusException("科目不存在");
        }
        student.setExamId(examId);
        student.setSchoolId(exam.getSchoolId());
        student.setSubjectName(subject.getName());
        student.setAbsent(false);
        student.setUpload(false);
        student.setException(false);
        student.setSliceCount(0);
        student.setSheetCount(0);
        student.setObjectivePageCount(0);
        student.setObjectiveScore(0d);
        student.setSubjectiveScore(0d);
        student.setSubjectiveStatus(SubjectiveStatus.UNMARK);
        student.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
        student.setInspectCount(0);
        ExamStudent old = checkExamNumber(student, new HashMap<String, ExamStudent>(),
                new HashMap<String, ExamStudent>());
        if (old != null) {
            throw new StatusException("准考证号" + student.getExamNumber() + "已经存在");
        } else {
            studentService.save(student);
            examService.updateExamUpdateTime(examId);
        }
        return resultOk();
    }
    @ApiOperation(value = "修改考生")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    @ResponseBody
    @Logging(menu = "修改考生", type = LogType.UPDATE)
    public ResultMessage update(HttpServletRequest request, ExamStudent student) {
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        ExamSubject subject = subjectService.find(examId, student.getSubjectCode());
        if (subject == null) {
            throw new StatusException("科目不存在");
        }
        if (student.getId() == null) {
            throw new StatusException("考生id不能为空");
        }
        ExamStudent previous = studentService.findById(student.getId());
        if (previous == null) {
            throw new StatusException("考生不存在");
        }
        if (!previous.isUpload()) {
            // 还未扫描上传的考生，才能修改准考证号、学习中心、科目
            previous.setCampusName(student.getCampusName());
            previous.setSubjectCode(subject.getCode());
            previous.setSubjectName(subject.getName());
            previous.setExamNumber(student.getExamNumber());
        }
        previous.setStudentCode(student.getStudentCode());
        previous.setName(student.getName());
        previous.setPackageCode(student.getPackageCode());
        previous.setCollege(student.getCollege());
        previous.setClassName(student.getClassName());
        previous.setTeacher(student.getTeacher());
        previous = studentService.save(previous);
        return resultOk();
    }

    private ExamStudent checkExamNumber(ExamStudent student, Map<String, ExamStudent> current,
            Map<String, ExamStudent> saveMap) {
        ExamStudent previous = saveMap.get(student.getExamNumber());

        if (previous != null && previous.getId() != null) {
            return previous;
        }
        if (current != null && !current.isEmpty()) {
            previous = current.get(student.getExamNumber());
        } else {
            previous = studentService.findByExamIdAndExamNumber(student.getExamId(), student.getExamNumber());
        }
        if (previous != null) {
            return previous;
        }
        current.put(student.getExamNumber(), student);
        return null;
    }

    @ApiOperation(value = "删除考生")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ResponseBody
    @Logging(menu = "删除考生", type = LogType.DELETE)
    public ResultMessage delete(HttpServletRequest request, @RequestParam Integer id) {
        ExamStudent student = studentService.findById(id);
        if (student == null) {
            throw new StatusException("考生不存在");
        }
        studentService.deleteById(id);
        markService.deleteByStudent(student);
        subjectService.updateUploadCount(student.getExamId(), student.getSubjectCode(), (int) studentService
                .countUploadedByExamIdAndSubjectCode(student.getExamId(), student.getSubjectCode()));
        RequestUtils.setLog(request, "删除成功，id:" + id);
        return resultOk();
    }

    @ApiOperation(value = "考生数据导入模版")
    @RequestMapping(value = "/template", method = RequestMethod.POST)
    public void importFileTemplate(HttpServletResponse response) {
        try {
            String fileName = "考生数据导入模板.xlsx";
            List<ExamStudent> list = Lists.newArrayList();
            list.add(new ExamStudent());
            new ExportExcel("考生数据", ExamStudent.class, 2).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "导入考生")
    @Logging(menu = "导入考生", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject importFile(HttpServletRequest request, MultipartFile file) {
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        try {
            int successNum = 0;
            int updateNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<ExamStudent> list = ei.getDataList(ExamStudent.class);
            List<ExamStudent> saveList = new LinkedList<ExamStudent>();
            List<ExamStudent> updateList = new LinkedList<ExamStudent>();
            Map<String, ExamStudent> saveMap = new HashMap<String, ExamStudent>();
            Map<String, ExamStudent> current = null;
            // 当数据库中已存在考生小于一定数量，则一次性取出所有考生，用于校验准考证号是否重复
            if (studentService.countByExamId(examId) < 10000) {
                current = new HashMap<String, ExamStudent>();
                List<ExamStudent> list2 = studentService.findByExamId(examId);
                for (ExamStudent s : list2) {
                    current.put(s.getExamNumber(), s);
                }
            }

            for (ExamStudent student : list) {
                if (StringUtils.isBlank(student.getExamNumber()) || StringUtils.isBlank(student.getName())
                        || StringUtils.isBlank(student.getSubjectCode())
                        || StringUtils.isBlank(student.getSubjectName()) || StringUtils.isBlank(student.getCollege())
                        || StringUtils.isBlank(student.getClassName()) || StringUtils.isBlank(student.getTeacher())
                        || StringUtils.isBlank(student.getStudentCode())) {
                    continue;
                }

                student.setExamId(examId);
                student.setSchoolId(exam.getSchoolId());
                student.setAbsent(false);
                student.setUpload(false);
                student.setException(false);
                student.setSliceCount(0);
                student.setSheetCount(0);
                student.setObjectivePageCount(0);
                student.setObjectiveScore(0d);
                student.setSubjectiveScore(0d);
                student.setSubjectiveStatus(SubjectiveStatus.UNMARK);
                student.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
                student.setInspectCount(0);
                ExamStudent previous = checkExamNumber(student, current, saveMap);
                if (previous != null && previous.getId() != null) {
                    student = updatePrevious(student, previous);
                    updateList.add(student);
                } else if (previous == null) {
                    saveList.add(student);
                }
                saveMap.put(student.getExamNumber(), student);
            }

            successNum = studentService.batchSave(saveList);
            updateNum = studentService.batchUpdate(updateList);
            examService.updateExamUpdateTime(examId);
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条用户");
            }
            String message = "已成功导入 " + (successNum + updateNum) + " 条用户";
            RequestUtils.setLog(request, message);
            return result(message + failureMsg);
        } catch (Exception e) {
            log.error("Batch import student error!", e);
            e.printStackTrace();
            throw new StatusException("导入考生失败！失败信息：" + e.getMessage());
        }
    }

    private ExamStudent updatePrevious(ExamStudent student, ExamStudent previous) {
        previous.setCollege(student.getCollege());
        previous.setClassName(student.getClassName());
        previous.setTeacher(student.getTeacher());
        previous.setExamRoom(student.getExamRoom());
        previous.setExamSite(student.getExamSite());
        previous.setRemark(student.getRemark());
        previous.setSubjectLevel(student.getSubjectLevel());
        previous.setSubjectCategory(student.getSubjectCategory());
        previous.setSubjectRemark(student.getSubjectRemark());
        return previous;
    }

    @ApiOperation(value = "导出考生")
    @Logging(menu = "导出考生", type = LogType.EXPORT)
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    public void exportFile(ExamStudentSearchQuery query, HttpServletRequest request, HttpServletResponse response) {
        try {
            int examId = getSessionExamId(request);
            query.setPageNumber(1);
            query.setPageSize(Integer.MAX_VALUE);
            query.setExamId(examId);
            query.addSort("id", Sort.Direction.ASC);
            query = studentService.findByQuery(query);
            for (ExamStudent student : query.getResult()) {
                ExamSubject subject = subjectService.find(student.getExamId(), student.getSubjectCode());
                student.setSubjectName(subject != null ? subject.getName() : "");
                student.setSubjectLevel(subject != null ? subject.getLevel() : "");
                student.setSubjectCategory(subject != null ? subject.getCategory() : "");
                student.setSubjectRemark(subject != null ? subject.getRemark() : "");
            }
            String fileName = "考生数据.xlsx";
            new ExportExcel("考生数据", ExamStudent.class).setDataList(query.getResult()).write(response, fileName)
                    .dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导出考生数据失败！" + e.getMessage());
        }
    }

    @ApiOperation(value = "缺考考生导入模版")
    @RequestMapping(value = "/absentTemplate", method = RequestMethod.POST)
    public void absentTemplate(HttpServletResponse response) {
        try {
            String fileName = "缺考考生导入模板.xlsx";
            List<ExamStudentVO> list = Lists.newArrayList();
            list.add(new ExamStudentVO());
            new ExportExcel("缺考考生", ExamStudentVO.class, 2).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "导入缺考名单")
    @Logging(menu = "导入缺考名单", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/absentImport", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject absentImportFile(HttpServletRequest request, MultipartFile file) {
        try {
            int examId = getSessionExamId(request);
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<ExamStudentVO> list = ei.getDataList(ExamStudentVO.class);
            studentService.clearManualAbsent(examId);
            for (ExamStudentVO studentVO : list) {
                if (StringUtils.isBlank(studentVO.getExamNumber())) {
                    continue;
                }
                ExamStudent student = studentService.findByExamIdAndExamNumber(examId, studentVO.getExamNumber());
                if (student != null) {
                    studentService.updateManualAbsent(examId, student.getExamNumber(), true);
                    successNum++;
                } else {
                    failureMsg.append("<br/>准考证号 " + studentVO.getExamNumber() + " 不存在; ");
                    failureNum++;
                }

            }
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条用户");
            }
            String message = "已成功导入 " + successNum + " 条用户";
            RequestUtils.setLog(request, message);
            return result(message + failureMsg);
        } catch (Exception e) {
            log.error("Batch import absentStudent error!", e);
            e.printStackTrace();
            throw new StatusException("导入缺考考生失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "违纪考生导入模版")
    @RequestMapping(value = "/breachTemplate", method = RequestMethod.POST)
    public void breachTemplate(HttpServletResponse response) {
        try {
            String fileName = "违纪考生导入模板.xlsx";
            List<ExamStudentVO> list = Lists.newArrayList();
            list.add(new ExamStudentVO());
            new ExportExcel("违纪考生", ExamStudentVO.class, 2).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "导入违纪名单")
    @Logging(menu = "导入违纪名单", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/breachImport", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject breachImportFile(HttpServletRequest request, MultipartFile file) {
        try {
            int examId = getSessionExamId(request);
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<ExamStudentVO> list = ei.getDataList(ExamStudentVO.class);
            for (ExamStudentVO studentVO : list) {
                if (StringUtils.isBlank(studentVO.getExamNumber())) {
                    continue;
                }
                ExamStudent student = studentService.findByExamIdAndExamNumber(examId, studentVO.getExamNumber());
                if (student != null) {
                    studentService.updateBreach(student.getId(), true);
                    successNum++;
                } else {
                    failureMsg.append("<br/>准考证号 " + studentVO.getExamNumber() + " 不存在; ");
                    failureNum++;
                }

            }
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条用户");
            }
            String message = "已成功导入 " + successNum + " 条用户";
            RequestUtils.setLog(request, message);
            return result(message + failureMsg);
        } catch (Exception e) {
            log.error("Batch import BreachStudent error!", e);
            e.printStackTrace();
            throw new StatusException("导入违纪考生失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "违纪重置")
    @Logging(menu = "违纪重置", type = LogType.UPDATE)
    @RequestMapping(value = "/updateBreach", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject updateBreach(HttpServletRequest request, Integer id) {
        JSONObject result = new JSONObject();
        ExamStudent student = studentService.findById(id);
        if (student != null && studentService.updateBreach(id, false)) {
            List<MarkGroup> groupList = groupService.findByExamAndSubjectAndStatus(student.getExamId(),
                    student.getSubjectCode(), MarkStatus.FINISH);
            for (MarkGroup markGroup : groupList) {
                groupService.updateStatus(student.getExamId(), student.getSubjectCode(), markGroup.getNumber(),
                        MarkStatus.FORMAL, MarkStatus.FINISH);
            }
            result.accumulate("message", "重置成功！");
            result.accumulate("success", true);
        } else {
            result.accumulate("message", "将违纪考生信息重置为正常时出错!");
            result.accumulate("success", false);
        }
        return result;
    }

    @ApiOperation(value = "多媒体考生上传模版")
    @RequestMapping(value = "/uploadTemplate", method = RequestMethod.POST)
    public void uploadTemplate(HttpServletResponse response) {
        try {
            String fileName = "多媒体考生上传导入模板.xlsx";
            List<UploadStudentVO> list = Lists.newArrayList();
            list.add(new UploadStudentVO());
            new ExportExcel("多媒体考生上传", UploadStudentVO.class, 2).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "导入多媒体考生")
    @RequestMapping(value = "/uploadImport", method = RequestMethod.POST)
    @Logging(menu = "导入多媒体考生", type = LogType.IMPORT_FILE)
    @ResponseBody
    public JSONObject uploadImportFile(HttpServletRequest request, MultipartFile file) {
        try {
            int examId = getSessionExamId(request);
            Exam exam = examService.findById(examId);
            if (exam.getType().equals(ExamType.SCAN_IMAGE)) {
                throw new StatusException("当前考试不是多媒体阅卷类型");
            }
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<UploadStudentVO> list = ei.getDataList(UploadStudentVO.class);
            for (UploadStudentVO studentVO : list) {
                if (StringUtils.isBlank(studentVO.getExamNumber())) {
                    continue;
                }
                ExamStudent student = studentService.findByExamIdAndExamNumber(examId, studentVO.getExamNumber());
                if (student != null) {
                    student.setUpload(true);
                    student.setAbsent(false);
                    student.setAnswers(null);
                    student.setBatchCode(null);
                    student.setSliceCount(0);
                    student.setSheetCount(0);
                    student.setObjectivePageCount(0);
                    student.setInspectCount(0);
                    if (student.getPaperType() != null && student.getPaperType().matches(PAPER_TYPES_REGEX)) {
                        student.setPaperType(StringUtils.trimToNull(studentVO.getPaperType()));
                    } else {
                        student.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
                    }
                    // 同步更新评卷任务
                    if (saveUploadStudent(student)) {
                        successNum++;
                    }
                } else {
                    failureMsg.append("<br/>准考证号 " + studentVO.getExamNumber() + " 不存在; ");
                    failureNum++;
                }

            }
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条用户");
            }
            return result("已成功导入 " + successNum + " 条用户" + failureMsg);
        } catch (Exception e) {
            log.error("Batch import BreachStudent error!", e);
            e.printStackTrace();
            throw new StatusException("导入上传考生失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "整理异常导入模版")
    @RequestMapping(value = "/collationLabelTemplate", method = RequestMethod.POST)
    public void collationLabelTemplate(HttpServletResponse response) {
        try {
            String fileName = "整理异常导入模板.xlsx";
            List<ExamStudentVO> list = Lists.newArrayList();
            list.add(new ExamStudentVO());
            new ExportExcel("整理异常", CollationLabelVO.class, 2).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "导入整理异常")
    @Logging(menu = "导入整理异常", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/collationLabelImport", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject collationLabelImportFile(HttpServletRequest request, MultipartFile file) {
        int examId = getSessionExamId(request);
        try {
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<CollationLabelVO> list = ei.getDataList(CollationLabelVO.class);
            List<CollationLabel> collationLabel = collationLabelService.list(examId);
            Map<String, CollationLabel> collationLabelMap = collationLabel.stream()
                    .collect(Collectors.toMap(CollationLabel::getCode, Function.identity()));
            studentService.clearCollationLabelCode(examId);
            for (CollationLabelVO studentVO : list) {
                if (StringUtils.isBlank(studentVO.getExamNumber())) {
                    continue;
                }
                ExamStudent student = studentService.findByExamIdAndExamNumber(examId, studentVO.getExamNumber());
                if (student != null && collationLabelMap.get(studentVO.getCollationLabelCode()) != null) {
                    studentService.updateCollationLabelCode(examId, student.getExamNumber(),
                            studentVO.getCollationLabelCode());
                    successNum++;
                } else {
                    failureMsg.append("<br/>准考证号 " + studentVO.getExamNumber() + " 不存在或异常代码 "
                            + studentVO.getCollationLabelCode() + " 不存在; ");
                    failureNum++;
                }

            }
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条");
            }
            String message = "已成功导入 " + successNum + " 条";
            RequestUtils.setLog(request, message);
            return result(message + failureMsg);
        } catch (Exception e) {
            log.error("Batch import CollationLabel error!", e);
            e.printStackTrace();
            throw new StatusException("导入整理异常失败！失败信息：" + e.getMessage());
        }
    }
}
